@@ -139,7 +139,7 @@ pulseaudio_button_init (PulseaudioButton *button)
   /* Preload icons */
   button->pixbuf_idx = 0;
   button->pixbufs = g_new0 (GdkPixbuf*, G_N_ELEMENTS (icons)-1);
-  g_signal_connect (G_OBJECT (button), "style_updated", G_CALLBACK (pulseaudio_button_update_icons), button); 
+  g_signal_connect (G_OBJECT (button), "style-updated", G_CALLBACK (pulseaudio_button_update_icons), button);
 
   /* Setup Gtk style */
   css_provider = gtk_css_provider_new ();
@@ -163,8 +163,6 @@ pulseaudio_button_init (PulseaudioButton *button)
 
   g_object_set (G_OBJECT (button), "has-tooltip", TRUE, NULL);
 
-  pulseaudio_button_update (button, TRUE);
-
   //button->align_box = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
   //gtk_container_add (GTK_CONTAINER (button), button->align_box);
   //gtk_widget_show (button->align_box);
@@ -201,22 +199,25 @@ pulseaudio_button_button_press (GtkWidget      *widget,
 {
   PulseaudioButton *button = PULSEAUDIO_BUTTON (widget);
 
-  if(event->button == 1) /* left button */
+  if(event->button == 1 && button->menu == NULL) /* left button */
     {
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-      if (button->menu == NULL)
-          button->menu = pulseaudio_menu_new (button->volume, button->config, widget);
+      button->menu = pulseaudio_menu_new (button->volume, button->config, widget);
+      gtk_menu_attach_to_widget (GTK_MENU (button->menu), widget, NULL);
+
       if (button->deactivate_id == 0)
+        {
           button->deactivate_id = g_signal_connect_swapped
             (GTK_MENU_SHELL (button->menu), "deactivate",
              G_CALLBACK (pulseaudio_button_menu_deactivate), button);
+        }
 
       gtk_menu_popup (GTK_MENU (button->menu),
                       NULL, NULL,
                       xfce_panel_plugin_position_menu, button->plugin,
                       //NULL, NULL,
-                      0,
-                      gtk_get_current_event_time ());
+                      1,
+                      event->time);
       return TRUE;
     }
 
@@ -260,6 +261,13 @@ pulseaudio_button_menu_deactivate (PulseaudioButton *button,
       button->deactivate_id = 0;
     }
 
+  if (button->menu != NULL)
+    {
+      gtk_menu_detach (GTK_MENU (button->menu));
+      gtk_menu_popdown (GTK_MENU (button->menu));
+      button->menu = NULL;
+    }
+
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
 }
 
@@ -299,6 +307,7 @@ pulseaudio_button_update (PulseaudioButton *button,
   gchar    *tip_text;
 
   g_return_if_fail (IS_PULSEAUDIO_BUTTON (button));
+  g_return_if_fail (IS_PULSEAUDIO_VOLUME (button->volume));
 
   volume = pulseaudio_volume_get_volume (button->volume);
   muted = pulseaudio_volume_get_muted (button->volume);
@@ -338,6 +347,7 @@ pulseaudio_button_set_size (PulseaudioButton *button,
   gint              width;
   gint              xthickness;
   gint              ythickness;
+  gint              size_old;
 
   g_return_if_fail (IS_PULSEAUDIO_BUTTON (button));
   g_return_if_fail (size > 0);
@@ -352,6 +362,7 @@ pulseaudio_button_set_size (PulseaudioButton *button,
   width = size - 2* MAX (xthickness, ythickness);
   /* Since symbolic icons are usually only provided in 16px we
    * try to be clever and use size steps */
+  size_old = button->icon_size;
   if (width <= 21)
       button->icon_size = 16;
   else if (width >=22 && width <= 29)
@@ -362,7 +373,8 @@ pulseaudio_button_set_size (PulseaudioButton *button,
       button->icon_size = width;
 
   gtk_widget_set_size_request (GTK_WIDGET (button), size, size);
-  pulseaudio_button_update_icons (button);
+  if (button->icon_size != size_old)
+    pulseaudio_button_update_icons (button);
 }
 
 
@@ -398,6 +410,8 @@ pulseaudio_button_new (PulseaudioPlugin *plugin,
     g_signal_connect_swapped (G_OBJECT (button->volume), "volume-changed",
                               G_CALLBACK (pulseaudio_button_volume_changed), button);
 
+  pulseaudio_button_update (button, TRUE);
+
   return GTK_WIDGET (button);
 }
 
