@@ -1,18 +1,18 @@
 /*  Copyright (c) 2014-2015 Andrzej <ndrwrdck@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License
+ *  as published by the Free Software Foundation; either version 2
+ *  of the License, or (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 
@@ -32,8 +32,10 @@
 
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 
+#include "pulseaudio-debug.h"
 #include "pulseaudio-plugin.h"
 #include "pulseaudio-config.h"
 #include "pulseaudio-volume.h"
@@ -56,6 +58,7 @@
 
 
 /* prototypes */
+static void             pulseaudio_plugin_init_debug                       (void);
 static void             pulseaudio_plugin_construct                        (XfcePanelPlugin       *plugin);
 static void             pulseaudio_plugin_free_data                        (XfcePanelPlugin       *plugin);
 static void             pulseaudio_plugin_show_about                       (XfcePanelPlugin       *plugin);
@@ -92,9 +95,6 @@ struct _PulseaudioPlugin
 
   /* config dialog builder */
   PulseaudioDialog    *dialog;
-
-  /* log file */
-  FILE                *logfile;
 };
 
 
@@ -124,9 +124,12 @@ pulseaudio_plugin_init (PulseaudioPlugin *pulseaudio_plugin)
 {
   g_log_set_always_fatal (G_LOG_LEVEL_ERROR);
 
-  pulseaudio_plugin->volume          = NULL;
-  pulseaudio_plugin->button          = NULL;
-  pulseaudio_plugin->logfile         = NULL;
+  /* initialize debug logging */
+  pulseaudio_plugin_init_debug ();
+  pulseaudio_debug("Pulseaudio Panel Plugin initialized");
+
+  pulseaudio_plugin->volume            = NULL;
+  pulseaudio_plugin->button            = NULL;
 }
 
 
@@ -144,6 +147,39 @@ pulseaudio_plugin_free_data (XfcePanelPlugin *plugin)
 
 
 
+static void
+pulseaudio_plugin_init_debug (void)
+{
+  const gchar  *debug_env;
+  gchar       **debug_domains;
+  gsize         i;
+  gchar        *message_debug_env;
+
+  /* enable debug output if the PANEL_DEBUG is set to "all" */
+  debug_env = g_getenv ("PANEL_DEBUG");
+  if ((debug_env != NULL) && (debug_env != '\0'))
+    {
+      debug_domains = g_strsplit (debug_env, ",", -1);
+      for (i = 0; debug_domains[i] != NULL; i++)
+        {
+          g_strstrip (debug_domains[i]);
+
+          if (g_str_equal (debug_domains[i], G_LOG_DOMAIN))
+            break;
+          else if (g_str_equal (debug_domains[i], "all"))
+            {
+              message_debug_env = g_strjoin (" ", G_LOG_DOMAIN, g_getenv ("G_MESSAGES_DEBUG"), NULL);
+              g_setenv ("G_MESSAGES_DEBUG", message_debug_env, TRUE);
+              g_free (message_debug_env);
+              break;
+            }
+        }
+      g_strfreev (debug_domains);
+    }
+}
+
+
+
 static void
 pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
 {
@@ -152,6 +188,7 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
   const gchar *auth[] =
     {
       "Andrzej Radecki <ndrwrdck@gmail.com>",
+      "Guido Berhoerster <guido+xfce@berhoerster.name>",
       NULL
     };
 
@@ -163,9 +200,9 @@ pulseaudio_plugin_show_about (XfcePanelPlugin *plugin)
                          "license",      xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
                          "version",      PACKAGE_VERSION,
                          "program-name", PACKAGE_NAME,
-                         "comments",     _("A panel plugin for controlling PulseAudio mixer."),
+                         "comments",     _("Adjust the audio volume of the PulseAudio sound system"),
                          "website",      "http://goodies.xfce.org/projects/panel-plugins/xfce4-pulseaudio-plugin",
-                         "copyright",    _("Copyright (c) 2014\n"),
+                         "copyright",    _("Copyright \xc2\xa9 2014 Andrzej Radecki\n"),
                          "authors",      auth,
                          NULL);
 
@@ -186,48 +223,6 @@ pulseaudio_plugin_configure_plugin (XfcePanelPlugin *plugin)
 
 
 
-static void
-pulseaudio_plugin_log_handler (const gchar    *domain,
-                               GLogLevelFlags  level,
-                               const gchar    *message,
-                               gpointer        data)
-{
-  PulseaudioPlugin *pulseaudio_plugin = PULSEAUDIO_PLUGIN (data);
-  gchar            *path;
-  const gchar      *prefix;
-
-  if (pulseaudio_plugin->logfile == NULL)
-    {
-      g_mkdir_with_parents (g_get_user_cache_dir (), 0755);
-      path = g_build_filename (g_get_user_cache_dir (), "xfce4-pulseaudio-plugin.log", NULL);
-      pulseaudio_plugin->logfile = fopen (path, "w");
-      g_free (path);
-    }
-
-  if (pulseaudio_plugin->logfile)
-    {
-      switch (level & G_LOG_LEVEL_MASK)
-        {
-        case G_LOG_LEVEL_ERROR:    prefix = "ERROR";    break;
-        case G_LOG_LEVEL_CRITICAL: prefix = "CRITICAL"; break;
-        case G_LOG_LEVEL_WARNING:  prefix = "WARNING";  break;
-        case G_LOG_LEVEL_MESSAGE:  prefix = "MESSAGE";  break;
-        case G_LOG_LEVEL_INFO:     prefix = "INFO";     break;
-        case G_LOG_LEVEL_DEBUG:    prefix = "DEBUG";    break;
-        default:                   prefix = "LOG";      break;
-        }
-
-      fprintf (pulseaudio_plugin->logfile, "%-10s %-25s %s\n", prefix, domain, message);
-      fflush (pulseaudio_plugin->logfile);
-    }
-
-  /* print log to the stdout */
-  if (level & G_LOG_LEVEL_ERROR || level & G_LOG_LEVEL_CRITICAL)
-    g_log_default_handler (domain, level, message, NULL);
-}
-
-
-
 static gboolean
 pulseaudio_plugin_size_changed (XfcePanelPlugin *plugin,
                                 gint             size)
@@ -263,7 +258,7 @@ pulseaudio_plugin_bind_keys (PulseaudioPlugin      *pulseaudio_plugin)
 {
   gboolean success;
   g_return_if_fail (IS_PULSEAUDIO_PLUGIN (pulseaudio_plugin));
-  g_debug ("Grabbing volume control keys");
+  pulseaudio_debug ("Grabbing volume control keys");
 
   success = (keybinder_bind (PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed, pulseaudio_plugin) &&
              keybinder_bind (PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed, pulseaudio_plugin) &&
@@ -280,7 +275,7 @@ static void
 pulseaudio_plugin_unbind_keys (PulseaudioPlugin      *pulseaudio_plugin)
 {
   g_return_if_fail (IS_PULSEAUDIO_PLUGIN (pulseaudio_plugin));
-  g_debug ("Releasing volume control keys");
+  pulseaudio_debug ("Releasing volume control keys");
 
   keybinder_unbind (PULSEAUDIO_PLUGIN_LOWER_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed);
   keybinder_unbind (PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY, pulseaudio_plugin_volume_key_pressed);
@@ -297,7 +292,7 @@ pulseaudio_plugin_volume_key_pressed (const char            *keystring,
   gdouble           volume_step       = pulseaudio_config_get_volume_step (pulseaudio_plugin->config) / 100.0;
   gdouble           new_volume;
 
-  g_debug ("%s pressed", keystring);
+  pulseaudio_debug ("%s pressed", keystring);
 
   if (strcmp (keystring, PULSEAUDIO_PLUGIN_RAISE_VOLUME_KEY) == 0)
     pulseaudio_volume_set_volume (pulseaudio_plugin->volume, MIN (MAX (volume + volume_step, 0.0), 1.0));
@@ -312,7 +307,7 @@ pulseaudio_plugin_mute_pressed (const char            *keystring,
 {
   PulseaudioPlugin *pulseaudio_plugin = PULSEAUDIO_PLUGIN (user_data);
 
-  g_debug ("%s pressed", keystring);
+  pulseaudio_debug ("%s pressed", keystring);
 
   pulseaudio_volume_toggle_muted (pulseaudio_plugin->volume);
 }
@@ -336,9 +331,6 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
   /* setup transation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-  /* log messages to a file */
-  g_log_set_default_handler (pulseaudio_plugin_log_handler, plugin);
-
   /* initialize xfconf */
   pulseaudio_plugin->config = pulseaudio_config_new (xfce_panel_plugin_get_property_base (plugin));
 
@@ -360,7 +352,8 @@ pulseaudio_plugin_construct (XfcePanelPlugin *plugin)
   pulseaudio_plugin->volume = pulseaudio_volume_new ();
 
   /* instantiate a button box */
-  pulseaudio_plugin->button = pulseaudio_button_new (pulseaudio_plugin->config,
+  pulseaudio_plugin->button = pulseaudio_button_new (pulseaudio_plugin,
+                                                     pulseaudio_plugin->config,
                                                      pulseaudio_plugin->volume);
   gtk_container_add (GTK_CONTAINER (plugin), GTK_WIDGET (pulseaudio_plugin->button));
   gtk_widget_show (GTK_WIDGET (pulseaudio_plugin->button));
